@@ -1,24 +1,14 @@
 package KinoSearch::Search::Similarity;
+use strict;
+use warnings;
 use KinoSearch::Util::ToolSet;
-use base qw( KinoSearch::Util::Class );
+use base qw( KinoSearch::Util::CClass );
 
 our %instance_vars = __PACKAGE__->init_instance_vars();
 
-sub new {
-    return _new();
-}
-
-# Provide a normalization factor for a field based on the square-root of the
-# number of terms in it, encoded into a single-byte.
-sub encode_lengthnorm {
-    my ( $self, $start ) = @_;
-    # a 0 is meaningless, but we have to prevent an illegal div by 0
-    return $start == 0 ? "\0" : $self->_float_to_byte( 1 / sqrt($start) );
-}
-
 # See _float_to_byte.
-sub encode_norm { $_[0]->_float_to_byte( $_[1] ) }
-sub decode_norm { $_[0]->_byte_to_float( $_[1] ) }
+*encode_norm = *_float_to_byte;
+*decode_norm = *_byte_to_float;
 
 # Calculate the Inverse Document Frequecy for one or more Term in a given
 # collection (the Searcher represents the collection).
@@ -41,8 +31,7 @@ sub idf {
     return $idf;
 }
 
-# Normalize a Query's weight so that it is comparable to other Queries.  Does
-# not affect ranking.
+# Normalize a Query's weight so that it is comparable to other Queries.
 sub query_norm {
     my ( $self, $sum_of_squared_weights ) = @_;
     return 0 if ( $sum_of_squared_weights == 0 );  # guard against div by zero
@@ -55,16 +44,32 @@ __END__
 
 __XS__
 
-MODULE = KinoSearch    PACKAGE = KinoSearch::Search::Similarity		
+MODULE = KinoSearch    PACKAGE = KinoSearch::Search::Similarity     
 
 Similarity*
-_new()
+new(...)
 CODE:
     RETVAL = Kino_Sim_new();
 OUTPUT: RETVAL
 
 =for comment
 
+Provide a normalization factor for a field based on the square-root of the
+number of terms in it.
+
+=cut
+
+float
+lengthnorm(sim, num_terms)
+    Similarity *sim;
+    U32         num_terms;
+CODE:
+    num_terms = num_terms < 100 ? 100 : num_terms;
+    RETVAL = (float)1 / sqrt(num_terms);
+OUTPUT: RETVAL
+
+=for comment
+
 Return a score factor based on the frequency of a term in a given document.
 The default implementation is sqrt(freq).  Other implementations typically
 produce ascending scores with ascending freqs, since the more times a doc
@@ -73,11 +78,11 @@ matches, the more relevant it is likely to be.
 =cut
 
 float
-tf(obj, freq)
-    Similarity *obj;
+tf(sim, freq)
+    Similarity *sim;
     U32         freq;
 CODE:
-    RETVAL = obj->tf(obj, freq);
+    RETVAL = sim->tf(sim, freq);
 OUTPUT: RETVAL
 
 
@@ -91,22 +96,22 @@ about one significant decimal digit.
 =cut
 
 SV*
-_float_to_byte( obj, f ) 
-    Similarity *obj;
+_float_to_byte(sim, f) 
+    Similarity *sim;
     float       f;
 PREINIT:
-    char        b;
+    char b;
 CODE:
-    b      = Kino_Sim_float2byte(obj, f);
+    b      = Kino_Sim_float2byte(sim, f);
     RETVAL = newSVpv(&b, 1);
 OUTPUT: RETVAL
 
 float
-_byte_to_float( obj, b ) 
-    Similarity *obj;
+_byte_to_float(sim, b) 
+    Similarity *sim;
     char        b;
 CODE:
-    RETVAL = Kino_Sim_byte2float(obj, b);
+    RETVAL = Kino_Sim_byte2float(sim, b);
 OUTPUT: RETVAL
 
 
@@ -119,32 +124,28 @@ knows how to use it.
 =cut
 
 SV*
-get_norm_decoder( obj )
-    Similarity *obj;
-PREINIT:
-    STRLEN      len;
+get_norm_decoder(sim)
+    Similarity *sim;
 CODE:
-    len = 256 * sizeof(float);
-    RETVAL = newSVpv((char*)obj->norm_decoder, len);
+    RETVAL = newSVpv( (char*)sim->norm_decoder, (256 * sizeof(float)) );
 OUTPUT: RETVAL
 
-void
-DESTROY(obj)
-    Similarity *obj;
-PPCODE:
-    Kino_Sim_destroy(obj);
-
 float
-coord(obj, overlap, max_overlap)
-    Similarity *obj;
+coord(sim, overlap, max_overlap)
+    Similarity *sim;
     U32         overlap;
     U32         max_overlap;
 CODE:
-    RETVAL = obj->coord(obj, overlap, max_overlap);
+    RETVAL = sim->coord(sim, overlap, max_overlap);
 OUTPUT: RETVAL
 
-    
+void
+DESTROY(sim)
+    Similarity *sim;
+PPCODE:
+    Kino_Sim_destroy(sim);
 
+    
 __H__
 
 #ifndef H_KINO_SIMILARITY
@@ -158,7 +159,7 @@ __H__
 typedef struct similarity {
     float  (*tf)(struct similarity*, float);
     float  (*coord)(struct similarity*, U32, U32);
-    float*   norm_decoder;
+    float   *norm_decoder;
 } Similarity;
 
 Similarity* Kino_Sim_new();
@@ -176,9 +177,9 @@ __C__
 
 Similarity*
 Kino_Sim_new() {
-    int           i;
-    unsigned char aUChar;
-    Similarity*   sim;
+    int            i;
+    unsigned char  aUChar;
+    Similarity    *sim;
 
     Kino_New(0, sim, 1, Similarity);
 
@@ -186,8 +187,7 @@ Kino_Sim_new() {
     Kino_New(0, sim->norm_decoder, 256, float);
     for (i = 0; i < 256; i++) {
         aUChar = i;
-        *(sim->norm_decoder + i) 
-            = Kino_Sim_byte2float(sim, (char)aUChar);
+        sim->norm_decoder[i] = Kino_Sim_byte2float(sim, (char)aUChar);
     }
 
     sim->tf    = Kino_Sim_default_tf;
@@ -289,7 +289,7 @@ Copyright 2005-2006 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.05_05.
+See L<KinoSearch|KinoSearch> version 0.09.
 
 =end devdocs
 =cut
